# Copyright (c) 2022 ANSYS, Inc. - Unauthorised use, distribution or duplication is prohibited.

import grpc

from ansys.report.engine.protos.common_pb2 import Empty
from ansys.report.engine.protos.report_pb2 import File, Header, Part, TitlePage
from ansys.report.engine.protos.report_pb2_grpc import ReportStub
from ansys.report.engine.server_creator import ServerCreator

from .report_logger import ReportLogger
from .report_part import ReportPart
from .report_title_page import ReportTitlePage


class Report:
    def __init__(self):
        self._server = ServerCreator().start_server()
        self._channel = grpc.insecure_channel(self._server.address)
        self._report_client = ReportStub(self._channel)
        self._logger = ReportLogger().configure_logging()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self._channel.close()
        del self._server
        del self._logger

    @ReportLogger.exception_handler
    def add_table_of_contents(self):
        """
        Add a table of contents to the report.
        """
        self._report_client.AddTableOfContents(Empty())

    @ReportLogger.exception_handler
    def add_header(self, header_text):
        """
        Add a page header text.

        Page headers are displayed on all pages except the first title page.
        """
        header = Header()
        header.header_text = header_text
        self._report_client.AddHeader(header)

    @ReportLogger.exception_handler
    def add_page_numbers(self):
        """
        Add page numbers to the footers.

        Page numbers are displayed on all pages except the first title page.
        """
        self._report_client.AddFooter(Empty())

    @ReportLogger.exception_handler
    def add_part(self, title):
        """
        Add a part.

        Parts are used to structure the report on a high level,
        e.g., to subdivide it into a main part and an appendix part.
        Each time a part is added the section numbering is reset.
        The part title is not shown as a heading in the report, and it is not listed in the table of contents.

        Parameters
        ----------
        title : str
           Title of the part

        Returns
        -------
        ReportPart
            Part created
        """
        new_part = Part()
        new_part.title = title
        id_of_new_part = self._report_client.AddPart(new_part)
        return ReportPart(self._report_client, id_of_new_part)

    @ReportLogger.exception_handler
    def add_title_page(self):
        """
        Add a title page to the report.

        A report can have multiple title pages (e.g. a cover page, a sign-off page, etc.).

        Returns
        -------
        ReportTitlePage
            Title Page created
        """
        new_title_page = TitlePage()

        id_of_new_title_page = self._report_client.AddTitlePage(new_title_page)
        return ReportTitlePage(self._report_client, id_of_new_title_page)

    @ReportLogger.exception_handler
    def export_pdf(self, path):
        """
        Export the report as a pdf.

        Parameters
        ----------
        path : str
            Absolute path of file (including .pdf)

        Returns
        -------
        bool
            True if the file exists; otherwise False
        """
        my_file = File()
        my_file.is_created = False
        my_file.path = path
        my_file = self._report_client.SavePdfReportDocument(my_file)
        return my_file.is_created
