# Copyright (c) 2022 ANSYS, Inc. - Unauthorised use, distribution or duplication is prohibited.

from ansys.report.engine.protos.report_pb2 import Size, Table, TableRowContent
from .report_units import ReportUnits


class ReportTable:
    def __init__(self, rows, column_width_list, caption=""):
        """
        Constructs all the necessary attributes for the table object.

        Parameters
        ----------
        rows : List[List[str]]
            List of rows of the table.
            The number of column entries in each row must be the same.
            The first row is treated as a header row.
            Example - [["Header 1", "Header 2"],["Content 1", "Content 2"]]
        column_width_list : List[float]
            List of column widths in centimeters.
            If the list has only a single value, this value applies to all the columns.
            Otherwise, there must be one value for each column of the table.
        caption : str
            Optional - table caption.
        """
        self.caption = caption
        self.rows = rows
        self.set_width_in_centimeter(column_width_list)

    @property
    def unit(self):
        """
        Read only - unit definition for column width.
        """
        return self._unit

    def set_width_in_centimeter(self, column_width_list):
        """
        Set the column widths in centimeter.

        Parameters
        ----------
        column_width_list: List[float]
            List of column widths in centimeters.
            If the list has only a single value, this value applies to all the columns.
            Otherwise, there must be one value for each column of the table.
        """
        self._unit = ReportUnits.CENTIMETER.value
        self.column_width_list = column_width_list

    def _to_proto_table(self, id):
        proto_table = Table()

        proto_table.parent_id.CopyFrom(id)

        proto_table.caption = self.caption

        for width in self.column_width_list:
            size = Size()
            size.unit = self._unit
            size.value = width
            proto_table.column_width.append(size)

        for row in self.rows:
            table_row_content = TableRowContent()
            table_row_content.content.extend(row)
            proto_table.table_content.table_row_content.append(table_row_content)

        return proto_table
